#include "nv_push_clcd97.h"

#include <stdio.h>

const char*
P_PARSE_NVCD97_MTHD(uint16_t idx)
{
    switch (idx) {
    case NVCD97_SET_ST_A:
        return "NVCD97_SET_ST_A";
    case NVCD97_SET_ST_B:
        return "NVCD97_SET_ST_B";
    case NVCD97_SET_ST_BLOCK_SIZE:
        return "NVCD97_SET_ST_BLOCK_SIZE";
    case NVCD97_SET_ST_ARRAY_PITCH:
        return "NVCD97_SET_ST_ARRAY_PITCH";
    case NVCD97_SET_ZT_FORMAT:
        return "NVCD97_SET_ZT_FORMAT";
    case NVCD97_SET_ST_SIZE_A:
        return "NVCD97_SET_ST_SIZE_A";
    case NVCD97_SET_ST_SIZE_B:
        return "NVCD97_SET_ST_SIZE_B";
    default:
        return "unknown method";
    }
}

void
P_DUMP_NVCD97_MTHD_DATA(FILE *fp, uint16_t idx, uint32_t data,
                         const char *prefix)
{
    UNUSED uint32_t parsed;
    switch (idx) {
    case NVCD97_SET_ST_A:
    
        parsed = (data >> 0) & ((1u << 8) - 1);
        fprintf(fp, "%s.OFFSET_UPPER = ", prefix);
        fprintf(fp, "(0x%x)\n", parsed);
        break;
    case NVCD97_SET_ST_B:
    
        parsed = data;
        fprintf(fp, "%s.OFFSET_LOWER = ", prefix);
        fprintf(fp, "(0x%x)\n", parsed);
        break;
    case NVCD97_SET_ST_BLOCK_SIZE:
    
        parsed = (data >> 0) & ((1u << 4) - 1);
        fprintf(fp, "%s.WIDTH = ", prefix);
        switch (parsed) {
        case NVCD97_SET_ST_BLOCK_SIZE_WIDTH_ONE_GOB:
            fprintf(fp, "ONE_GOB\n");
            break;
        default:
            fprintf(fp, "0x%x\n", parsed);
            break;
        }
    
        parsed = (data >> 4) & ((1u << 4) - 1);
        fprintf(fp, "%s.HEIGHT = ", prefix);
        switch (parsed) {
        case NVCD97_SET_ST_BLOCK_SIZE_HEIGHT_ONE_GOB:
            fprintf(fp, "ONE_GOB\n");
            break;
        case NVCD97_SET_ST_BLOCK_SIZE_HEIGHT_TWO_GOBS:
            fprintf(fp, "TWO_GOBS\n");
            break;
        case NVCD97_SET_ST_BLOCK_SIZE_HEIGHT_FOUR_GOBS:
            fprintf(fp, "FOUR_GOBS\n");
            break;
        case NVCD97_SET_ST_BLOCK_SIZE_HEIGHT_EIGHT_GOBS:
            fprintf(fp, "EIGHT_GOBS\n");
            break;
        case NVCD97_SET_ST_BLOCK_SIZE_HEIGHT_SIXTEEN_GOBS:
            fprintf(fp, "SIXTEEN_GOBS\n");
            break;
        case NVCD97_SET_ST_BLOCK_SIZE_HEIGHT_THIRTYTWO_GOBS:
            fprintf(fp, "THIRTYTWO_GOBS\n");
            break;
        default:
            fprintf(fp, "0x%x\n", parsed);
            break;
        }
    
        parsed = (data >> 8) & ((1u << 4) - 1);
        fprintf(fp, "%s.DEPTH = ", prefix);
        switch (parsed) {
        case NVCD97_SET_ST_BLOCK_SIZE_DEPTH_ONE_GOB:
            fprintf(fp, "ONE_GOB\n");
            break;
        default:
            fprintf(fp, "0x%x\n", parsed);
            break;
        }
        break;
    case NVCD97_SET_ST_ARRAY_PITCH:
    
        parsed = data;
        fprintf(fp, "%s.V = ", prefix);
        fprintf(fp, "(0x%x)\n", parsed);
        break;
    case NVCD97_SET_ZT_FORMAT:
    
        parsed = (data >> 0) & ((1u << 5) - 1);
        fprintf(fp, "%s.V = ", prefix);
        switch (parsed) {
        case NVCD97_SET_ZT_FORMAT_V_Z16:
            fprintf(fp, "Z16\n");
            break;
        case NVCD97_SET_ZT_FORMAT_V_Z24S8:
            fprintf(fp, "Z24S8\n");
            break;
        case NVCD97_SET_ZT_FORMAT_V_X8Z24:
            fprintf(fp, "X8Z24\n");
            break;
        case NVCD97_SET_ZT_FORMAT_V_S8Z24:
            fprintf(fp, "S8Z24\n");
            break;
        case NVCD97_SET_ZT_FORMAT_V_S8:
            fprintf(fp, "S8\n");
            break;
        case NVCD97_SET_ZT_FORMAT_V_ZF32:
            fprintf(fp, "ZF32\n");
            break;
        case NVCD97_SET_ZT_FORMAT_V_ZF32_X24S8:
            fprintf(fp, "ZF32_X24S8\n");
            break;
        default:
            fprintf(fp, "0x%x\n", parsed);
            break;
        }
    
        parsed = (data >> 8) & ((1u << 1) - 1);
        fprintf(fp, "%s.STENCIL_IS_SEPARATE = ", prefix);
        switch (parsed) {
        case NVCD97_SET_ZT_FORMAT_STENCIL_IS_SEPARATE_FALSE:
            fprintf(fp, "FALSE\n");
            break;
        case NVCD97_SET_ZT_FORMAT_STENCIL_IS_SEPARATE_TRUE:
            fprintf(fp, "TRUE\n");
            break;
        default:
            fprintf(fp, "0x%x\n", parsed);
            break;
        }
        break;
    case NVCD97_SET_ST_SIZE_A:
    
        parsed = (data >> 0) & ((1u << 28) - 1);
        fprintf(fp, "%s.WIDTH = ", prefix);
        fprintf(fp, "(0x%x)\n", parsed);
        break;
    case NVCD97_SET_ST_SIZE_B:
    
        parsed = (data >> 0) & ((1u << 18) - 1);
        fprintf(fp, "%s.HEIGHT = ", prefix);
        fprintf(fp, "(0x%x)\n", parsed);
        break;
    default:
        fprintf(fp, "%s.VALUE = 0x%x\n", prefix, data);
        break;
    }
}
